@@ -1,3 +1,3 @@
-from . import atlas, checkatlas, multiqc, folders
+from . import atlas, checkatlas, folders, multiqc
 
 __all__ = ["atlas", "checkatlas", "multiqc", "folders"]
